@@ -30,8 +30,14 @@ test_that("NA and 'NA' compare differently", {
 test_that("multiline comparison", {
   expect_snapshot({
     compare_character("A\nthe apple is red\nC\n", "A\nthe apple was red\nC\n")
-    compare_character("A\nthe apple is red and green\nC\n", "A\nthe apple is red\nC\n")
-    compare_character("A\nthe apple is red and green\nC\n", "A\nI like bananas\nC\n")
+    compare_character(
+      "A\nthe apple is red and green\nC\n",
+      "A\nthe apple is red\nC\n"
+    )
+    compare_character(
+      "A\nthe apple is red and green\nC\n",
+      "A\nI like bananas\nC\n"
+    )
 
     "trailing newlines are correctly compared"
     compare("x\n", "x")
@@ -42,14 +48,25 @@ test_that("multi-element multi-line comparisons get indices", {
   expect_snapshot({
     compare(c("a", "b", "c\nd"), c("a", "b", "c\ne"))
   })
-
 })
 
 test_that("show elementwise differences of random permutations", {
   expect_snapshot({
-    compare(letters[1:15], letters[c(14, 4, 12, 11, 13, 3, 10, 5, 1, 7, 9, 15, 6, 8, 2)], max_diffs = Inf)
-    compare(letters[1:15], letters[c(3, 13, 6, 10, 11, 9, 4, 5, 15, 2, 12, 14, 8, 7, 1)], max_diffs = Inf)
-    compare(letters[1:15], letters[c(12, 13, 1, 2, 5, 6, 11, 15, 10, 14, 9, 7, 3, 4, 8)], max_diffs = Inf)
+    compare(
+      letters[1:15],
+      letters[c(14, 4, 12, 11, 13, 3, 10, 5, 1, 7, 9, 15, 6, 8, 2)],
+      max_diffs = Inf
+    )
+    compare(
+      letters[1:15],
+      letters[c(3, 13, 6, 10, 11, 9, 4, 5, 15, 2, 12, 14, 8, 7, 1)],
+      max_diffs = Inf
+    )
+    compare(
+      letters[1:15],
+      letters[c(12, 13, 1, 2, 5, 6, 11, 15, 10, 14, 9, 7, 3, 4, 8)],
+      max_diffs = Inf
+    )
   })
 })
 
@@ -80,15 +97,25 @@ test_that("numeric comparison", {
     compare_numeric(x, x + c(-1, 0, 1) * 1e-6)
     compare_numeric(x, x + c(-1, 0, 1) * 1e-7)
     compare_numeric(x, x + c(-1, 0, 1) * 1e-8)
-    compare_numeric(x, x + c(-1, 0, 1) * 1e-8, tolerance = 0)
-    compare_numeric(x, x + c(-1, 0, 1) * 1e-9, tolerance = 0)
-    compare_numeric(x, x + c(-1, 0, 1) * 1e-10, tolerance = 0)
+    compare_numeric(x, x + c(-1, 0, 1) * 1e-8, tolerance = NULL)
+    compare_numeric(x, x + c(-1, 0, 1) * 1e-9, tolerance = NULL)
+    compare_numeric(x, x + c(-1, 0, 1) * 1e-10, tolerance = NULL)
 
     "unequal length"
     compare_numeric(c(1, 2, NA), c(1, 2 + 1e-7, NA, 3))
   })
 })
 
+test_that("tolerance is used in display of differences", {
+  x <- c(1, 2, 3)
+  y <- x + c(1e-9, 1e-9, 1)
+
+  expect_snapshot({
+    compare_numeric(x, y)
+    compare_numeric(x, y, tolerance = NULL)
+  })
+})
+
 test_that("NAs are shown regardless of position", {
   expect_snapshot({
     compare(c(NA, 1, 2), c(1, 2))
@@ -97,8 +124,11 @@ test_that("NAs are shown regardless of position", {
   })
 })
 
-test_that("informative difference between NA and NaN", {
-  expect_snapshot(compare_numeric(NA_real_, NaN))
+test_that("informative difference between NA and NaN when tolerance set", {
+  expect_snapshot({
+    compare_numeric(NA_real_, NaN)
+    compare_numeric(NA_real_, NaN, tolerance = NULL)
+  })
 })
 
 test_that("numeric comparison works on factors", {
@@ -178,17 +208,22 @@ test_that("logical comparisons minimise extraneous diffs", {
 })
 
 test_that("min_digits correctly computed digits needed for comparison", {
-  expect_equal(min_digits(0.21, 0.23), 2)
-  expect_equal(min_digits(1.93, 1.92), 2)
+  expect_equal(min_digits(-0.5, 0.5), 1)
+  expect_equal(min_digits(-0.9090909, 0.9090909), 1)
+  expect_equal(min_digits(0.21, 0.23), 3)
+  expect_equal(min_digits(1.93, 1.92), 3)
 
-  expect_equal(min_digits(1, 1.1), 1)
-  expect_equal(min_digits(1, 1.01), 2)
-  expect_equal(min_digits(1, 1.001), 3)
-  expect_equal(min_digits(1, 1.0001), 4)
-  expect_equal(min_digits(1, 1.00001), 5)
-  expect_equal(min_digits(1, 1.000001), 6)
-  expect_equal(min_digits(1, 1.0000001), 7)
-  expect_equal(min_digits(1, 1.00000001), 8)
+  expect_equal(min_digits(1, 1.1), 2)
+  expect_equal(min_digits(1, 1.01), 3)
+  expect_equal(min_digits(1, 1.001), 4)
+  expect_equal(min_digits(1, 1.0001), 5)
+  expect_equal(min_digits(1, 1.00001), 6)
+  expect_equal(min_digits(1, 1.000001), 7)
+  expect_equal(min_digits(1, 1.0000001), 8)
+  expect_equal(min_digits(1, 1.00000001), 9)
   expect_equal(min_digits(1, 1.000000001), 9)
-  expect_equal(min_digits(1, 1.0000000001), 10)
+  expect_equal(min_digits(1, 1.0000000001), 9)
+
+  expect_equal(min_digits(1, 1.000000001, tolerance = NULL), 10)
+  expect_equal(min_digits(1, 1.0000000001, tolerance = NULL), 11)
 })
